CON

  _CLKMODE        = XTAL1 + PLL16X
  _XINFREQ        = 5_000_000

  'Propeller Memory Card Assignments
  DI              = 0 'IO0 / DI / MOSI / CD Pin
  DO              = 1 'IO1 / DO / MISO Pin
  WP              = 2 'IO2 / WP Pin
  HOLD            = 3 'IO3 / HOLD Pin
  SD_CS           = 4 'CS Pin For MicroSD Card Memory
  FLASH_CS        = 5 'CS Pin For Flash Memory
  SRAM_CS         = 6 'CS Pin For SRAM Memory
  CLK             = 7 'CLK Pin

  'Full Duplex Serial
  RX              = 31   
  TX              = 30  
  COMM_MODE       = 0
  COMM_BAUD_RATE  = 115_200

  'FAT Engine Assignment for unused pins or RTC. Please refer to the
  'FAT16/FAT32 Full File System Driver Documentation for proper usage
  NEG_ONE         = -1
   
VAR

  'The following variable holds the G-Code file line characters.  It has been
  'established that the G-Code file used as a guide in this code, had a maximum
  'character count of 72.  To be on the safe side, the maximum file line size
  'has been increased to 100. 
  BYTE bChBuffer[100]

  'The following variables are intended to store G-Code into various fields.
  'For more information about these fields, please refer to RepRap G Code
  'Fields at: http://reprap.org/wiki/G-code .  Additionally, these are the
  'only known fields to be supported by the KISSlicer.  Please note that the
  'maximum character count may change in the future, because I am not exactly
  'sure on bFieldElement_8 maximum character count, but support for 7 characters
  'should be sufficient.  The maximum character count should support a build
  'volume area of 999.99mm(X) X 999.99mm(Y) X 999.99mm(Z).  
  BYTE bFieldElement_0[2] 'G Field
  BYTE bFieldElement_1[3] 'M Field
  BYTE bFieldElement_2[1] 'T Field
  BYTE bFieldElement_3[3] 'S Field
  BYTE bFieldElement_4[7] 'X Field - Floating Point
  BYTE bFieldElement_5[7] 'Y Field - Floating Point
  BYTE bFieldElement_6[7] 'Z Field - Floating Point
  BYTE bFieldElement_7[7] 'F Field - Floating Point
  BYTE bFieldElement_8[7] 'E Field - Floating Point

  'The following G-Code commands are currently the only known (by me) commands
  'supported by the KISSlicer, therefore these are the only G-Code commands
  'that I will currently support.
  {
  G1: Controlled Move With E, F, X, Y, And Z Fields.
  G20: Set Measurement Units To Inches With No Fields
  G21: Set Measurement Units To Millimeters With No Fields
  G28: Home Axes With No Fields
  G90: Set Absolute Positioning With No Fields
  G91: Set Relative Positioning With No Fields
  M82: Set E Codes Absolute With No Fields
  M83: Set E Codes Relative With No Fields
  M104: Set Extruder Temperature And No Wait With S Field
  M106: Fan On With No Fields
  M109: Set Extruder Temperature And Wait With S Field
  M107: Fan Off With No Fields
  M140: Set Bed Temperature With S Field
  T0: Select Tool With No Fields 
  }

OBJ

  SDCard:       "SD-MMC_FATEngine.spin"
  Memory:       "SPI Memory Driver"
  Terminal:     "FullDuplexSerial"

PUB Main

  'Start FullDuplexSerial
  Terminal.Start(RX, TX, COMM_MODE, COMM_BAUD_RATE)
  WAITCNT(CNT + (1 * CLKFREQ))

  'Start FATEngine
  SDCard.fatEngineStart(DO, CLK, DI, SD_CS, NEG_ONE, NEG_ONE, NEG_ONE, NEG_ONE, NEG_ONE)

  'Mount Partition
  SDCard.mountPartition(0)

  'Open G-Code File "CAL_HOLD.TXT" for reading
  SDCard.openFile(STRING("CAL_HOLD.TXT"), "R")

  REPEAT

    'Fill bChBuffer with 0s
    BYTEFILL(@bChBuffer, 0, 100)

    'Grab file lines that are not comments or blank
    GetNextValidFileLine

    IF(bChBuffer[0] == 0)
      QUIT

    'Fill in the various field elements
    FillFieldElements

    'Display the various field seperations
    Terminal.Str(STRING("G="))    
    Terminal.Str(@bFieldElement_0)
    Terminal.Tx(32) {Space}

    Terminal.Str(STRING("M="))
    Terminal.Str(@bFieldElement_1)
    Terminal.Tx(32) {Space}

    Terminal.Str(STRING("T="))
    Terminal.Str(@bFieldElement_2)
    Terminal.Tx(32) {Space}

    Terminal.Str(STRING("S="))
    Terminal.Str(@bFieldElement_3)
    Terminal.Tx(32) {Space}

    Terminal.Str(STRING("X="))
    Terminal.Str(@bFieldElement_4)
    Terminal.Tx(32) {Space}

    Terminal.Str(STRING("Y="))
    Terminal.Str(@bFieldElement_5)
    Terminal.Tx(32) {Space}

    Terminal.Str(STRING("Z="))
    Terminal.Str(@bFieldElement_6)
    Terminal.Tx(32) {Space}

    Terminal.Str(STRING("F="))
    Terminal.Str(@bFieldElement_7)
    Terminal.Tx(32) {Space}

    Terminal.Str(STRING("E="))
    Terminal.Str(@bFieldElement_8)
    Terminal.Tx(32) {Space}

    'Seperate G-Code field values line by line
    Terminal.Tx(13) {Carriage Return}

  'Unmount Partition
  SDCard.unmountPartition

  'Stop FATEngine
  SDCard.fatEngineStop

  WAITCNT(CNT + (1 * CLKFREQ))

  'Stop FullDuplexSerial  
  Terminal.Stop

PUB GetNextValidFileLine

  REPEAT

    'Grab a file line as a string  
    SDCard.readString(@bChBuffer, 99)

    'Keep repeating while the following conditions are met,
    'because we don't want or need these lines beginning with
    'these characters.
    IF(bChBuffer[0] <> 59 {; Character} AND bChBuffer[0] <> 13 {Carriage Return} AND bChBuffer[0] <> 10 {Line Feed})
      QUIT

PUB FillFieldElements | Char, Index, Field_Element, ValueIndex

  'Empty the field elements of any possible data
  BYTEFILL(@bFieldElement_0, 0, 2)
  BYTEFILL(@bFieldElement_1, 0, 3)
  BYTEFILL(@bFieldElement_2, 0, 1)
  BYTEFILL(@bFieldElement_3, 0, 3)
  BYTEFILL(@bFieldElement_4, 0, 7)
  BYTEFILL(@bFieldElement_5, 0, 7)
  BYTEFILL(@bFieldElement_6, 0, 7)
  BYTEFILL(@bFieldElement_7, 0, 7)
  BYTEFILL(@bFieldElement_8, 0, 7)

  Index := 0
  ValueIndex := 0

  'Parse file line into seperate field values.  Various fields will remain empty
  REPEAT

    Char := bChBuffer[Index]

      'Exclude these characters from becoming part of are field entries
      IF(Char == 13 {Carriage Return} OR Char == 10 {Line Feed})
        QUIT

      ELSEIF(Char == 32 {Space})
        Index++
        ValueIndex := 0
        NEXT

      ELSEIF(Char == 71 {G Character})

        'Set current field element
        Field_Element := 0
        Index++
        NEXT

      ELSEIF(Char == 77 {M Character})

        'Set current field element
        Field_Element := 1
        Index++
        NEXT

      ELSEIF(Char == 84 {T Character})

        'Set current field element
        Field_Element := 2
        Index++
        NEXT

      ELSEIF(Char == 83 {S Character})

        'Set current field element
        Field_Element := 3
        Index++
        NEXT

      ELSEIF(Char == 88 {X Character})

        'Set current field element
        Field_Element := 4
        Index++
        NEXT

      ELSEIF(Char == 89 {Y Character})

        'Set current field element
        Field_Element := 5
        Index++
        NEXT

      ELSEIF(Char == 90 {Z Character})

        'Set current field element
        Field_Element := 6
        Index++
        NEXT

      ELSEIF(Char == 70 {F Character})

        'Set current field element
        Field_Element := 7
        Index++
        NEXT

      ELSEIF(Char == 69 {E Character})

        'Set current field element
        Field_Element := 8
        Index++
        NEXT

      ELSE

        IF(Field_Element == 0)

          'G Field element character
          bFieldElement_0[ValueIndex] := Char
          ValueIndex++
          Index++
          NEXT

        ELSEIF(Field_Element == 1)

          'M Field element character
          bFieldElement_1[ValueIndex] := Char
          ValueIndex++
          Index++
          NEXT

        ELSEIF(Field_Element == 2)

          'T Field element character
          bFieldElement_2[ValueIndex] := Char
          ValueIndex++
          Index++
          NEXT

        ELSEIF(Field_Element == 3)

          'S Field element character
          bFieldElement_3[ValueIndex] := Char
          ValueIndex++
          Index++
          NEXT

        ELSEIF(Field_Element == 4)

          'X Field element character
          bFieldElement_4[ValueIndex] := Char
          ValueIndex++
          Index++
          NEXT

        ELSEIF(Field_Element == 5)

          'Y Field element character
          bFieldElement_5[ValueIndex] := Char
          ValueIndex++
          Index++
          NEXT

        ELSEIF(Field_Element == 6)

          'Z Field element character
          bFieldElement_6[ValueIndex] := Char
          ValueIndex++
          Index++
          NEXT

        ELSEIF(Field_Element == 7)

          'F Field element character
          bFieldElement_7[ValueIndex] := Char
          ValueIndex++
          Index++
          NEXT

        ELSEIF(Field_Element == 8)

          'E Field element character
          bFieldElement_8[ValueIndex] := Char
          ValueIndex++
          Index++
          NEXT